@@ -7,16 +7,9 @@
  Copyright Henrik Bengtsson, 2007
  **************************************************************************/
 #include <Rdefines.h>
-#include "types.h"
-#include "utils.h"
-
-
-#define METHOD anyMissing
-#define METHOD_NAME anyMissing_internal
-#define RETURN_TYPE int
-#define ARGUMENTS_LIST SEXP x, void *idxs, R_xlen_t nidxs
-
-#include "templates-gen-vector.h"
+#include "000.types.h"
+#include "000.utils.h"
+#include "anyMissing_lowlevel.h"
 
 
 SEXP anyMissing(SEXP x, SEXP idxs) {
