@@ -5,40 +5,75 @@
  Copyright Henrik Bengtsson, 2014
  **************************************************************************/
 #include <Rdefines.h>
-#include "types.h"
+#include "000.types.h"
+
+void indexByRow_i(int nrow, int ncol, int *idxs_ptr, R_xlen_t nidxs, int *ans_ptr) {
+  R_xlen_t i, idx, n_max;
+  int col, row;
+  
+  if (idxs_ptr == NULL) {
+    row = 1;
+    col = 0;
+    for (i = 0; i < nidxs; i++) {
+      ans_ptr[i] = row + col * nrow;
+      col++;
+      if (col == ncol) {
+        row++;
+        col = 0;
+      }
+    }
+  } else {
+    n_max = (R_xlen_t)nrow * (R_xlen_t)ncol;
+    for (i = 0; i < nidxs; i++) {
+      idx = idxs_ptr[i] - 1;
+      if (idx < 0) {
+        error("Argument 'idxs' may only contain positive indices: %d", idx + 1);
+      } if (idx >= n_max) {
+        error("Argument 'idxs' contains indices larger than %d: %d",
+	      n_max, idx + 1);
+      }
+      col = idx / ncol;
+      row = idx % ncol;
+      idx = col + nrow * row + 1;
+      ans_ptr[i] = idx;
+    }
+  }
+} // indexByRow_i()
+
 
 SEXP indexByRow(SEXP dim, SEXP idxs) {
   SEXP ans;
-  int nr, nc;
-  int *ans_ptr, *idxs_ptr;
-  R_xlen_t i, idx, n, nidxs;
-  int col, row;
-  int d;
-  double nd = 1.0;
+  int d, i;
+  R_xlen_t nidxs;
+  double n_max;
+  int *idxs_ptr;
 
   /* Argument 'dim': */
   if (!isInteger(dim) || xlength(dim) != 2) {
     error("Argument 'dim' must be an integer vector of length two.");
   }
 
+  n_max = 1.0;
   for (i = 0; i < xlength(dim); i++) {
     d = INTEGER(dim)[i];
     if (d < 0) {
       error("Argument 'dim' specifies a negative value: %d", d);
     }
-    nd *= d;
+    n_max *= d;
 #ifndef LONG_VECTOR_SUPPORT
-    if (nd > R_INT_MAX) {
-      error("Argument 'dim' specifies too many elements: %.g > %d", nd, R_INT_MAX);
+    if (n_max > R_INT_MAX) {
+      error("Argument 'dim' (%d,%d) specifies a matrix that has more than 2^31-1 elements: %d", INTEGER(dim)[1], INTEGER(dim)[2], n_max);
     }
 #endif
+    if (n_max > R_INT_MAX) {
+      error("Argument 'dim' (%d,%d) specifies a matrix that has more than 2^31-1 elements: %d", INTEGER(dim)[1], INTEGER(dim)[2], n_max);
+    }
   }
-  n = (R_xlen_t)nd;
 
   /* Argument 'idxs': */
   if (isNull(idxs)) {
     idxs_ptr = NULL;
-    nidxs = 0;
+    nidxs = (R_xlen_t)n_max;
   } else if (isVectorAtomic(idxs)) {
     idxs_ptr = INTEGER(idxs);
     nidxs = xlength(idxs);
@@ -49,45 +84,13 @@ SEXP indexByRow(SEXP dim, SEXP idxs) {
     error("Argument 'idxs' must be NULL or a vector.");
   }
 
-  nr = INTEGER(dim)[0];
-  nc = INTEGER(dim)[1];
+  PROTECT(ans = allocVector(INTSXP, nidxs));
 
-  if (idxs_ptr == NULL) {
-    PROTECT(ans = allocVector(INTSXP, n));
-    ans_ptr = INTEGER(ans);
-
-    row = 1;
-    col = 0;
-    for (i = 0; i < n; i++) {
-      ans_ptr[i] = row + col*nr;
-      col++;
-      if (col == nc) {
-        row++;
-        col = 0;
-      }
-    }
-
-    UNPROTECT(1);
-  } else {
-    PROTECT(ans = allocVector(INTSXP, nidxs));
-    ans_ptr = INTEGER(ans);
-
-    for (i = 0; i < nidxs; i++) {
-      // idxs <- idxs - 1
-      // cols <- idxs %/% dim[2L]
-      // rows <- idxs  %% dim[2L]
-      // cols + dim[1L]*rows + 1L
-      idx = idxs_ptr[i] - 1;
-      col = idx / nc;
-      row = idx - nc*col;
-      row = idx % nc;
-      idx = col + nr*row + 1;
-      ans_ptr[i] = idx;
-    }
-
-    UNPROTECT(1);
-  }
+  indexByRow_i(INTEGER(dim)[0], INTEGER(dim)[1],
+               idxs_ptr, nidxs, INTEGER(ans));
 
+  UNPROTECT(1);
+  
   return(ans);
 } // indexByRow()
 
